<?php

namespace App\baseClasses;
use App\controllers\KCBookAppointmentWidgetController;
use App\controllers\KCServiceController;
class WidgetHandler extends KCBase {

	public function init() {
        add_shortcode('bookAppointment', [$this, 'bookAppointmentWidget']);
        add_shortcode('patientDashboard', [$this, 'patientDashboardWidget']);
        add_shortcode('kivicareBookAppointment', [$this, 'kivicareBookAppointmentWidget']);
    }

    public function bookAppointmentWidget ($param) {
        ob_start();
        if(empty($this->getLoginUserRole()) || $this->getLoginUserRole() === 'kiviCare_patient'){

            // sanitize parameters of shortcode
            $temp = $this->sanitizeShortCodeData($param,'bookAppointmentWidget');
            $doctor_id = $temp['doctor'];
            $clinic_id = $temp['clinic'];
            $service_id = $temp['service'];
            $user_id = get_current_user_id();

            // condition if clinic id provide in shortcode parameter or $_Get is available
            if($clinic_id != 0){
                if($this->checkIfPreSelectClinicExists($clinic_id)){
                    echo esc_html__('Selected clinic not available', 'kc-lang');
                    return ob_get_clean();
                }
            }else{
                // condition if anyy clinic is available in system and if only 1 available preselect it
                $kiviCareclinic = $this->checkIfKiviCareAnyClinicAvailable($clinic_id);
                if($kiviCareclinic['status']){
                    echo esc_html__('No clinic available', 'kc-lang');
                    return ob_get_clean();
                }else{
                    $clinic_id = $kiviCareclinic['clinic'];
                }
            }

            // condition if clinic id provide in shortcode parameter or $_Get is available
            if($doctor_id != 0){
                if($clinic_id == 0){
                    echo esc_html__('Please Provide clinic id', 'kc-lang');
                    return ob_get_clean();
                }
                if($this->checkIfPreSelectDoctorExists($doctor_id)){
                    echo esc_html__('Select Doctor Not available', 'kc-lang');
                    return ob_get_clean();
                }
            }else{
                // condition if anyy doctor is available in system and if only 1 available preselect it
                $kiviCareDoctor = $this->checkIfKiviCareAnyDoctorAvailable($doctor_id);
                if($kiviCareDoctor['status']){
                    echo esc_html__('No Doctor  available', 'kc-lang');
                    return ob_get_clean();
                }else{
                    $doctor_id = $kiviCareDoctor['doctor'];
                }
            }

            $this->shortcodeScript('bookAppointmentWidget');
            echo "<div id='app' class='kivi-care-appointment-booking-container kivi-widget' >
            <book-appointment-widget v-bind:user_id='$user_id' v-bind:doctor_id='$doctor_id' v-bind:clinic_id='$clinic_id' v-bind:service_id='$service_id' >
            </book-appointment-widget></div>";
        }else{
            echo esc_html__('Admin can not view the widget. Only patients can view the widget. Please open this page in incognito mode or use another browser without an admin login.', 'kc-lang');
        }
        return ob_get_clean();
    }

    public function patientDashboardWidget () {
        ob_start();
        if(empty($this->getLoginUserRole()) || $this->getLoginUserRole() === 'kiviCare_patient'){
            $this->shortcodeScript('patientDashboardWidget');
            echo "<div id='app' class='kivi-care-patient-dashboard-container kivi-widget' ><patient-dashboard-widget></patient-dashboard-widget></div>";
        }else{
            echo esc_html__('Admin can not view the widget. Only patients can view the widget. Please open this page in incognito mode or use another browser without an admin login.', 'kc-lang');
        }
        return ob_get_clean();
    }

    public function kivicareBookAppointmentWidget($param){
        ob_start();
        if (empty($this->getLoginUserRole()) || $this->getLoginUserRole() === 'kiviCare_patient') {
            // sanitize parameters of shortcode
            $temp = $this->sanitizeShortCodeData($param,'kivicareBookAppointmentWidget');
            $shortcode_doctor_id = $temp['doctor'];
            $shortcode_clinic_id = $temp['clinic'];
            $shortcode_service_id = $temp['service'];

            // condition if clinic id provide in shortcode parameter or $_Get is available
            if($shortcode_clinic_id != 0){
                if($this->checkIfPreSelectClinicExists($shortcode_clinic_id)){
                    echo esc_html__('Selected clinic not available', 'kc-lang');
                    return ob_get_clean();
                }
            }else{
                // condition if anyy clinic is available in system and if only 1 available preselect it
                $kiviCareclinic = $this->checkIfKiviCareAnyClinicAvailable($shortcode_clinic_id);
                if($kiviCareclinic['status']){
                    echo esc_html__('No clinic available', 'kc-lang');
                    return ob_get_clean();
                }else{
                    $shortcode_clinic_id = $kiviCareclinic['clinic'];
                }
            }

            // condition if doctor id provide in shortcode parameter or $_Get is available
            if($shortcode_doctor_id != 0){
                if($shortcode_clinic_id == 0){
                    echo esc_html__('Please Provide clinic id', 'kc-lang');
                    return ob_get_clean();
                }
                if($this->checkIfPreSelectDoctorExists($shortcode_doctor_id)){
                    echo esc_html__('Select Doctor Not available', 'kc-lang');
                    return ob_get_clean();
                }
                $shortcode_doctor_id_single = !(strpos($shortcode_doctor_id, ',') !== false);
            }else{
                // condition if anyy doctor is available in system and if only 1 available preselect it
                $kiviCareDoctor = $this->checkIfKiviCareAnyDoctorAvailable($shortcode_doctor_id);
                if($kiviCareDoctor['status']){
                    echo esc_html__('No Doctor  available', 'kc-lang');
                    return ob_get_clean();
                }else{
                    $shortcode_doctor_id_single = $kiviCareDoctor['single'];
                    $shortcode_doctor_id = $kiviCareDoctor['doctor'];
                }
            }

            $this->shortcodeScript('kivicareBookAppointmentWidget');
            require KIVI_CARE_DIR . 'app/baseClasses/bookAppointment/bookAppointment.php';

        } else {

            echo esc_html__('Admin can not view the widget. Only patients can view the widget. Please open this page in incognito mode or use another browser without an admin login.', 'kc-lang');

        }

        return ob_get_clean();

    }

    public function shortcodeScript($type){
        wp_enqueue_style('kc_font_awesome');
        if($type === 'kivicareBookAppointmentWidget'){
            wp_enqueue_style('kc_book_appointment');
            wp_enqueue_script('kc_axios');
            wp_enqueue_script('kc_flatpicker');
            wp_enqueue_style('kc_flatpicker');
            if(kcGetSingleWidgetSetting('widget_print')){
                wp_enqueue_script('kc_print');
            }
        }else{
            kcAppendLanguageInHead();
            wp_enqueue_style('kc_front_app_min_style');
            wp_dequeue_style( 'stylesheet' );
            wp_enqueue_script('kc_custom');
            wp_enqueue_script('kc_front_js_bundle');
        }
    }

    public function sanitizeShortCodeData($param,$type){

        $shortcode_doctor_id = 0;
        $shortcode_clinic_id = 0;
        $shortcode_service_id = -1;
        if(isset($param['doctor_id']) && !empty($param['doctor_id'])){
            if($type === 'kivicareBookAppointmentWidget'){
                $shortcode_doctor_id = isset($param['doctor_id']) ? sanitize_text_field($param['doctor_id']) : 0;
            }else{
                $shortcode_doctor_id = isset($param['doctor_id']) ? '"'.sanitize_text_field($param['doctor_id']).'"' : 0;
            }
        }elseif(isset($_GET['doctor_id']) && !empty($_GET['doctor_id'])){
            $shortcode_doctor_id = sanitize_text_field(wp_unslash($_GET['doctor_id']));
        }

        if(isset($param['clinic_id']) && !empty($param['clinic_id'])){
            $shortcode_clinic_id = isset($param['clinic_id']) ? sanitize_text_field($param['clinic_id']) : 0;
        }elseif(isset($_GET['clinic_id']) && !empty($_GET['clinic_id'])){
            $shortcode_clinic_id = sanitize_text_field(wp_unslash($_GET['clinic_id']));
        }

        if(isset($param['service_id']) && !empty($param['service_id'])){
            $shortcode_service_id = isset($param['service_id']) ? sanitize_text_field($param['service_id']) : -1;
        }
        return [
            'doctor' => $shortcode_doctor_id,
            'clinic' => $shortcode_clinic_id,
            'service' => $shortcode_service_id
        ];
    }

    public function checkIfPreSelectClinicExists($clinic_id){
        global $wpdb;
        $status = false;
        // condition if clinic id provide in shortcode parameter or $_Get
        if($clinic_id != 0 ){
            $clinic_id = (int)$clinic_id;
            if(!isKiviCareProActive()){
                if($clinic_id != kcGetDefaultClinicId()){
                    $status = true;
                }
            }else{
                $clinic_count = $wpdb->get_var("SELECT count(*) FROM {$wpdb->prefix}kc_clinics WHERE id = {$clinic_id} AND status = 1");
                if($clinic_count == 0){
                    $status = true;
                }
            }
        }
        return $status;
    }

    public function checkIfPreSelectDoctorExists($doctor_id){
        global $wpdb;
        $status = false;
        if($doctor_id != 0 && !(strpos($doctor_id, ',') !== false)){
            $doctor_id = (int)$doctor_id;
            $userExists = $wpdb->get_var("SELECT COUNT(*) FROM {$wpdb->base_prefix}users WHERE ID = {$doctor_id} AND user_status = 0");
            if($userExists == 0){
                $status = true;

            }
        }
        return $status;
    }

    public function checkIfKiviCareAnyDoctorAvailable($doctor_id){
        $status = false;
        $single_doctor = false;
        $args['role'] = $this->getDoctorRole();
        $args['user_status'] = '0';
        $allDoctor = get_users($args);
        if(empty($allDoctor)){
            $status = true;
        }else if($doctor_id == 0 && count($allDoctor) === 1 ){
            $single_doctor = true;
            foreach ($allDoctor as $doc){
                $doctor_id = $doc->ID;
            }
        }

        return [
            'status' => $status,
            'doctor' => $doctor_id,
            'single' => $single_doctor
        ];
    }

    public function checkIfKiviCareAnyClinicAvailable($clinic_id){
        global $wpdb;
        $status = false;
        $clinic_count = $wpdb->get_var("SELECT count(*) FROM {$wpdb->prefix}kc_clinics WHERE status = 1");

        //if no clinic data is found return
        if($clinic_count == 0){
            $status = true;
        }

        // if pro active and clinic id is not provide in shortcode or $_GET
        if(isKiviCareProActive() && $clinic_id === 0){
            // if only one clinic is available default selected
            if($clinic_count == 1){
                $clinic_id = $wpdb->get_var("SELECT id FROM {$wpdb->prefix}kc_clinics WHERE status = 1");
            }
        }

        if(!isKiviCareProActive()){
            $clinic_id = kcGetDefaultClinicId();
        }
        return ['status' => $status , 'clinic' => $clinic_id];
    }
}


