<?php

namespace App\Controllers;


use App\baseClasses\KCBase;
use App\baseClasses\KCRequest;
use App\models\KCBill;
use App\models\KCBillItem;
use App\models\KCClinic;
use App\models\KCPatientEncounter;
use App\models\KCService;
use App\models\KCServiceDoctorMapping;
use App\models\KCAppointmentServiceMapping;
use Exception;

class KCPatientBillController extends KCBase {

	public $db;

	public $bill;
	/**
	 * @var KCRequest
	 */
	private $request;

	public function __construct() {

		global $wpdb;

		$this->db = $wpdb;

		$this->bill = new KCBill();

		$this->request = new KCRequest();

	}

	public function index() {

		if ( ! kcCheckPermission( 'patient_bill_list' ) ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => esc_html__('You don\'t have a permission to access', 'kc-lang'),
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();

		$total_rows = count( $request_data );
		if ( ! isset( $request_data['encounter_id'] ) ) {
			echo json_encode( [
				'status'  => false,
				'message' => esc_html__('Encounter not found', 'kc-lang'),
				'data'    => []
			] );
			wp_die();
		}

		$encounter_id = (int)$request_data['encounter_id'];
		$bills        = $this->bill->get_by( [ 'encounter_id' => $encounter_id ] );

		if ( ! count( $bills ) ) {
			echo json_encode( [
				'status'     => false,
				'message'    => esc_html__('No bills records found', 'kc-lang'),
				'data'       => [],
				'total_rows' => $total_rows
			] );
			wp_die();
		}

		echo json_encode( [
			'status'  => true,
			'message' => esc_html__('Medical records', 'kc-lang'),
			'data'    => $bills
		] );
	}

	public function save() {
		$active_domain =$this->getAllActivePlugin();
		if ( ! kcCheckPermission( 'patient_bill_add' ) ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => esc_html__('You don\'t have a permission to access', 'kc-lang'),
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();

		

		$rules = [
			'encounter_id'  => 'required',
			'total_amount'  => 'required',
			'discount'      => 'required',
			'actual_amount' => 'required',
			'billItems'     => 'required'
		];

		$errors = kcValidateRequest( $rules, $request_data );

		if ( count( $errors ) ) {
			echo json_encode( [
				'status'  => false,
				'message' => $errors[0]
			] );
			die;
		}

        $request_data['encounter_id'] = (int)$request_data['encounter_id'];
		$patient_encounter = ( new KCPatientEncounter )->get_by( [ 'id' => $request_data['encounter_id'] ], '=', true );
		if ( empty( $patient_encounter ) ) {
			echo json_encode( [
				'status'  => false,
				'message' => esc_html__("No encounter found", 'kc-lang')
			] );
			die;
		}
		$user_id = get_current_user_id();
		$clinic_mapping = (new KCClinic())->get_by([ 'clinic_admin_id' => $user_id]);
		$temp = [
			'title'         => $request_data['title'],
			'encounter_id'  => $request_data['encounter_id'],
			'clinic_id'  => isset($patient_encounter->clinic_id) ? $patient_encounter->clinic_id : 1 ,
			'total_amount'  => $request_data['total_amount'],
			'discount'      => $request_data['discount'],
			'actual_amount' => $request_data['actual_amount'],
			'status'        => $request_data['status'],
			'payment_status' => isset($request_data['payment_status']['id'])? $request_data['payment_status']['id'] : 'unpaid',
			'appointment_id' => isset($request_data['appointment_id']) ? (int)$request_data['appointment_id']: 0,
		];

		

		if ( ! isset( $request_data['id'] ) ) {
            //hook when bill generate
            do_action('kc_encounter_bill_generate',$temp);
			$temp['created_at'] = current_time( 'Y-m-d H:i:s' );
			$status             = $this->bill->insert( $temp );
			$bill_id            = $status;

		} else {
            //hook when bill paid
            do_action('kc_encounter_bill_update',$temp);
			$bill_id = $request_data['id'];
			$status  = $this->bill->update( $temp, array( 'id' => (int)$request_data['id'] ) );

		}
		if ( isset( $request_data['billItems'] ) && count( $request_data['billItems'] ) ) {
			// insert bill items
            foreach ( $request_data['billItems'] as $key => $bill_item ) {
				if ((int) $bill_item['item_id']['price'] === 0 ) {
					$service_object = new KCService();
					$service        = $service_object->get_by( [
						'type' => 'bill_service',
						'name' => strtolower( $bill_item['item_id']['label'] )
					], '=', true );
					// here if service not exist then add into service table
					// $service = stripslashes($service);
					
					if ( $service ) {
						$bill_item['item_id']['id'] = $service->id;
					} else {
                        $new_service['type']        = 'bill_service';
						$new_service['name']        = strtolower( $bill_item['item_id']['id'] );
						$new_service['status']      = 1;
						$new_service['created_at'] = current_time( 'Y-m-d H:i:s' );
						$service_id                 = $service_object->insert( $new_service );
						$bill_item['item_id']['id'] = $service_id;
						$service_doctor_mapping = new KCServiceDoctorMapping();
						$service_appoiment_mapping = new KCAppointmentServiceMapping();
						if ( isset( $request_data['doctor_id'] ) && $request_data['doctor_id'] > 0 ) {
							if ($service_id) {
								$service_doctor_mapping->insert([
									'service_id' => (int)$service_id,
									'clinic_id'  => kcGetDefaultClinicId(),
									'doctor_id'  => (int)$request_data['doctor_id'],
									'charges'    => $bill_item['price']
								]);
							}
							$message            = __('Service has been saved successfully','kc-lang');
						}
						if(isset( $request_data['appointment_id'] ) && $request_data['appointment_id'] > 0 && $request_data['appointment_id']!= null){
							if ($service_id) {
								$service_appoiment_mapping->insert([
									'service_id' => (int)$service_id,
									'appointment_id'=> (int)$request_data['appointment_id'],
									'status'=> 1
								]);
							}
						}
					}

				}
				
                $_temp = [
					'bill_id' => (int)$bill_id,
					'price'   => (int)$bill_item['price'],
					'qty'     => (int)$bill_item['qty'],
					'item_id' => (int)$bill_item['item_id']['id'],
				];

				$bill_item_object = new KCBillItem();
				
				if ( ! isset( $bill_item['id'] ) ) {
					$_temp['created_at'] = current_time( 'Y-m-d H:i:s' );
					$bill_item_object->insert( $_temp );
				} else {
					$bill_item_object->update( $_temp, array( 'id' => (int)$bill_item['id'] ) );
				}
			}
		}
		
		if(!empty($request_data['payment_status']) && !empty($request_data['payment_status']['id']) && $request_data['payment_status']['id'] == 'paid'){
            //hook when bill paid
            do_action('kc_encounter_bill_paid',$temp);
			( new KCPatientEncounter() )->update( [ 'status' => '0' ], [ 'id' => $request_data['encounter_id'] ] );
			if($active_domain === $this->kiviCareProOnName()){
                if(kcCheckSmsOptionEnable()){
                    $response = apply_filters('kcpro_send_sms', [
                        'type' => 'encounter_close',
                        'encounter_id' => $request_data['encounter_id'],
                        'patient_id'=>$patient_encounter->patient_id
                    ]);
                }
			}

			echo json_encode( [
				'status'  => true,
				'message' => esc_html__('Bill has been generated successfully', 'kc-lang')
			] );  die;
			
		}else{
            //hook when bill unpaid
            do_action('kc_encounter_bill_unpaid',$temp);
			( new KCPatientEncounter() )->update( [ 'status' => '1' ], [ 'id' => $request_data['encounter_id'] ] );
		}

		echo json_encode( [
			'status'  => true,
			'message' => esc_html__('Encounter saved successfully', 'kc-lang')
		] );

	}

	public function edit() {

		if ( ! kcCheckPermission( 'patient_bill_edit' ) || ! kcCheckPermission( 'patient_bill_view' ) ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => esc_html__('You don\'t have a permission to access', 'kc-lang'),
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();

		try {

			if ( ! isset( $request_data['id'] ) ) {
				throw new Exception( esc_html__('Data not found', 'kc-lang'), 400 );
			}

			$id = (int)$request_data['id'];

			$results = $this->bill->get_by( [ 'id' => $id ], '=', true );

			if ( $results ) {

				$temp = [
					'id'            => $results->id,
					'title'         => $results->title,
					'encounter_id'  => $results->encounter_id,
					'total_amount'  => $results->total_amount,
					'discount'      => $results->discount,
					'actual_amount' => $results->actual_amount,
					'status'        => $results->status,
					'billItems'     => []
				];

				$billItems = ( new KCBillItem )->get_by( [ 'bill_id' => $results->id ], '=', false );

				if ( count( $billItems ) ) {
					foreach ( $billItems as $item ) {
						$service = ( new KCService )->get_by( [ 'id' => (int)$item->item_id ], '=', true );

						$temp['billItems'][] = [
							'bill_id' => $item,
							'id'      => $item->id,
							'price'   => $item->price,
							'qty'     => $item->qty,
							'item_id' => [
								'id'    => $item->item_id,
								'label' => $service->name,
								'price' => $service->price,
							],
						];
					}
				}


				echo json_encode( [
					'status'  => true,
					'message' => 'Bill item',
					'data'    => $temp
				] );
			} else {
				throw new Exception( esc_html__('Data not found', 'kc-lang'), 400 );
			}


		} catch ( Exception $e ) {

			$code    = $e->getCode();
			$message = $e->getMessage();

			header( "Status: $code $message" );

			echo json_encode( [
				'status'  => false,
				'message' => $message
			] );
		}
	}


	public function details() {

		if ( ! kcCheckPermission( 'patient_bill_view' ) ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => esc_html__('You don\'t have a permission to access', 'kc-lang'),
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();

        global $wpdb;

		try {

			$id           = isset( $request_data['id'] ) ? (int)$request_data['id'] : 0;
			$encounter_id = isset( $request_data['encounter_id'] ) ? (int)$request_data['encounter_id'] : 0;

			if ( $encounter_id !== 0 ) {
                $results =  $wpdb->get_row("SELECT * FROM {$wpdb->prefix}kc_bills WHERE encounter_id={$encounter_id}");
			} else {
                $results = $wpdb->get_row("SELECT * FROM {$wpdb->prefix}kc_bills WHERE id={$id}");
			}


			if (!empty($results)) {

				$patient_encounter = ( new KCPatientEncounter )->get_by( [ 'id' => (int)$results->encounter_id ], '=', true );
				$clinic            = ( new KCClinic )->get_by( [ 'id' => (int)$patient_encounter->clinic_id ], '=', true );

				$patient_data = kcGetUserData($patient_encounter->patient_id);

				$temp = [
					'id'               => $results->id,
					'title'            => $results->title,
					'encounter_id'     => $results->encounter_id,
					'total_amount'     => $results->total_amount,
					'discount'         => $results->discount,
					'actual_amount'    => $results->actual_amount,
					'status'           => $results->status,
					'payment_status'=>$results->payment_status,
					'created_at'       => $results->created_at,
					'billItems'        => [],
					'patientEncounter' => $patient_encounter,
					'clinic'           => $clinic,
					'patient'          => [
						'id' => $patient_data->ID,
						'display_name' => $patient_data->display_name,
						'gender' => isset($patient_data->basicData->gender) ? $patient_data->basicData->gender : "",
						'dob' => isset($patient_data->basicData->dob) ? $patient_data->basicData->dob : ""
					]
				];

				$billItems = ( new KCBillItem )->get_by( [ 'bill_id' => (int)$results->id ], '=', false );
				if (!empty($billItems)) {
					foreach ( $billItems as $item ) {
						$service = ( new KCService )->get_by( [ 'id' => (int)$item->item_id ], '=', true );
						
						$temp['billItems'][] = [
							'bill_id' => $item,
							'id'      => $item->id,
							'price'   => $item->price,
							'qty'     => $item->qty,
							'item_id' => [
								'id'    => $item->item_id,
								'label' => $service->name,
								'price' => $service->price,
							],
						];
					}
				}
				echo json_encode( [
					'status'  => true,
					'message' => esc_html__('Bill item', 'kc-lang'),
					'data'    => $temp
				] );

			} else {
				echo json_encode( [
					'status'  => true,
					'message' => esc_html__('Bill not found', 'kc-lang'),
					'data'    => []
				] );
			}



		} catch ( Exception $e ) {

			$code    = $e->getCode();
			$message = $e->getMessage();

			header( "Status: $code $message" );

			echo json_encode( [
				'status'  => false,
				'message' => $message
			] );
		}
	}

	public function delete() {

		if ( ! kcCheckPermission( 'patient_bill_delete' ) ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => esc_html__('You don\'t have a permission to access', 'kc-lang'),
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();

		try {

			if ( ! isset( $request_data['id'] ) ) {
				throw new Exception( esc_html__('Data not found', 'kc-lang'), 400 );
			}

			$id = (int)$request_data['id'];

            //hook when bill delete
            do_action('kc_encounter_bill_delete',$id);

			( new KCBillItem() )->delete( [ 'bill_id' => $id ] );

			$results = ( new KCBill() )->delete( [ 'id' => $id ] );

			if ( $results ) {
				echo json_encode( [
					'status'  => true,
					'message' => esc_html__('Bill item has been deleted successfully', 'kc-lang'),
				] );
			} else {
				throw new Exception( esc_html__('Bill item delete failed', 'kc-lang'), 400 );
			}


		} catch ( Exception $e ) {

			$code    = $e->getCode();
			$message = $e->getMessage();

			header( "Status: $code $message" );

			echo json_encode( [
				'status'  => false,
				'message' => $message
			] );
		}
	}

	public function deleteBillItem() {

		$request_data = $this->request->getInputs();

		try {

			if ( ! isset( $request_data['bill_item_id'] ) ) {
				throw new Exception( esc_html__('Data not found', 'kc-lang'), 400 );
			}

			$id = (int)$request_data['bill_item_id'];

			$results = ( new KCBillItem() )->delete( [ 'id' => $id ] );

			if ( $results ) {
				echo json_encode( [
					'status'  => true,
					'message' => esc_html__('Bill item has been deleted successfully', 'kc-lang'),
				] );
			} else {
				throw new Exception( esc_html__('Bill item delete failed', 'kc-lang'), 400 );
			}


		} catch ( Exception $e ) {

			$code    = $e->getCode();
			$message = $e->getMessage();

			header( "Status: $code $message" );

			echo json_encode( [
				'status'  => false,
				'message' => $message
			] );
		}
	}

	public function updateStatus() {

		if ( ! kcCheckPermission( 'patient_bill_edit' ) ) {
			echo json_encode( [
				'status'      => false,
				'status_code' => 403,
				'message'     => esc_html__('You don\'t have a permission to access', 'kc-lang'),
				'data'        => []
			] );
			wp_die();
		}

		$request_data = $this->request->getInputs();

		try {

			if ( ! isset( $request_data['id'] ) ) {
				throw new Exception( esc_html__('Data not found', 'kc-lang'), 400 );
			}

            $request_data['id'] = (int)$request_data['id'];
			$results = $this->bill->update( [ 'status' => 1 ], array( 'id' => $request_data['id'] ) );
            //hook when bill generate
            do_action('kc_encounter_bill_status_update', $request_data['id'] );

			if ( $results ) {
				echo json_encode( [
					'status'  => true,
					'message' => esc_html__('Payment status has been updated successfully', 'kc-lang'),
				] );
			} else {
				throw new Exception( esc_html__('update status process failed', 'kc-lang'), 400 );
			}


		} catch ( Exception $e ) {

			$code    = $e->getCode();
			$message = $e->getMessage();

			header( "Status: $code $message" );

			echo json_encode( [
				'status'  => false,
				'message' => $message
			] );
		}
	}

	public function sendPaymentLink(){
		
		echo json_encode( [
			'data' => [],
			'status'  => true,
			'message' => __('link ready to access','kc-lang')
		] );
	}

    public function printEncounterBillDetail(){
        $request_data = $this->request->getInputs();

        if(!empty($request_data['id']) && !empty($request_data['data'])){
            $patient_encounter_table = $this->db->prefix . 'kc_patient_encounters';
            $clinics_table           = $this->db->prefix . 'kc_clinics';
            $users_table             = $this->db->base_prefix . 'users';
            $encouter_id = (int)$request_data['id'];
            $query = "SELECT {$patient_encounter_table}.*,
                       doctors.display_name  AS doctor_name,
                       doctors.user_email AS doctor_email,    
                       patients.display_name AS patient_name,
                       patients.user_email AS patient_email,
                       {$clinics_table}.* 
                    FROM  {$patient_encounter_table}
                       LEFT JOIN {$users_table} doctors
                              ON {$patient_encounter_table}.doctor_id = doctors.id
                      LEFT JOIN {$users_table} patients
                              ON {$patient_encounter_table}.patient_id = patients.id
                       LEFT JOIN {$clinics_table}
                              ON {$patient_encounter_table}.clinic_id = {$clinics_table}.id
                    WHERE {$patient_encounter_table}.id = {$encouter_id}";

            $encounter = $this->db->get_row( $query);

            if ( !empty( $encounter ) ) {
                $encounter->medical_history = '';
                $encounter->prescription = '';
                $basic_data = get_user_meta((int)$encounter->doctor_id, 'basic_data', true);
                $basic_data = json_decode($basic_data);
                foreach ($basic_data->qualifications as $q) {
                    $qualifications[] = $q->degree;
                    $qualifications[] = $q->university;
                }
                $patient_basic_data = json_decode(get_user_meta((int)$encounter->patient_id, 'basic_data', true));
                $encounter->patient_gender = !empty($patient_basic_data->gender)
                    ? ($patient_basic_data->gender === 'female'
                        ? 'F' : 'M') : '';
                $encounter->patient_address = (!empty($patient_basic_data->address) ? $patient_basic_data->address : '');
                $encounter->patient_city = (!empty($patient_basic_data->city) ? $patient_basic_data->city : '');
                $encounter->patient_state = (!empty($patient_basic_data->state) ? $patient_basic_data->state : '');
                $encounter->patient_country = (!empty($patient_basic_data->country) ? $patient_basic_data->country : '');
                $encounter->patient_postal_code = (!empty($patient_basic_data->postal_code) ? $patient_basic_data->postal_code : '');
                $encounter->contact_no = (!empty($patient_basic_data->mobile_number) ? $patient_basic_data->mobile_number : '');
                $encounter->patient_add = $encounter->patient_address . ',' . $encounter->patient_city
                    . ',' . $encounter->patient_state . ',' . $encounter->patient_country . ',' . $encounter->patient_postal_code;
                $years = $months = $days = 0;
                $encounter->date = current_time('Y-m-d');
                if (!empty($patient_basic_data->dob)) {
                    $diff = abs(strtotime(current_time('Y-m-d')) - strtotime($patient_basic_data->dob));
                    $years = floor($diff / (365 * 60 * 60 * 24));
                    $months = floor(($diff - $years * 365 * 60 * 60 * 24) / (30 * 60 * 60 * 24));
                    $days = floor(($diff - $years * 365 * 60 * 60 * 24 - $months * 30 * 60 * 60 * 24) / (60 * 60 * 24)) . esc_html__('Days', 'kc-lang');
                    $years = $years . esc_html__(' Years', 'kc-lang');
                    $months = $months . esc_html__('Days', 'kc-lang');
                }
                $encounter->patient_age = $years === '0 Years' ? ($months === '0 Months' ? $days : $months) : $years;
                $encounter->qualifications = !empty($qualifications) ? '(' . implode(", ", $qualifications) . ')' : '';
                $encounter->clinic_address = "#{$encounter->clinic_id } ,{$encounter->address } ,{$encounter->city}";
                $default_clinic_log = KIVI_CARE_DIR_URI .'assets/images/kc-demo-img.png';
                $encounter->clinic_logo = !empty($encounter->profile_image) ? wp_get_attachment_url($encounter->profile_image) : $default_clinic_log;
                $encounter->billItems = !empty($request_data['data']['billItems']) ? $request_data['data']['billItems'] : [];
                $encounter->payment_status = !empty($request_data['data']['payment_status']) ? $request_data['data']['payment_status'] :'';
                $encounter->bill_created = !empty($request_data['data']['created_at']) ? $request_data['data']['created_at'] :'';
                $encounter->actual_amount = !empty($request_data['data']['actual_amount']) ? $request_data['data']['actual_amount'] :'';
                $encounter->total_amount = !empty($request_data['data']['total_amount']) ? $request_data['data']['total_amount'] :'';
                $encounter->discount = !empty($request_data['data']['discount']) ? $request_data['data']['discount'] :'0';
            }
            echo json_encode([
                'data' => kcPrescriptionHtml($encounter,$encouter_id,'bill_detail'),
                'status' => true
            ]);
            wp_die();
        }else{
            echo json_encode( [
                'data' => [],
                'status'  => false,
                'message' => __('Encounter Id not found','kc-lang')
            ] );
        }
    }
}